unit findexp1;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Buttons, StdCtrls, ExptIntf, ToolIntf, FileCtrl, ExtCtrls;

type
  TFindExpDialog = class(TForm)
    SearchFileName: TEdit;
    Label1: TLabel;
    FoundList: TListBox;
    SearchCriteria: TRadioGroup;
    CancelButton: TBitBtn;
    LoadButton: TBitBtn;
    FoundListLabel: TLabel;
    DriveComboBox: TDriveComboBox;
    SearchButton: TBitBtn;
    procedure SearchCriteriaClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure LoadButtonClick(Sender: TObject);
    procedure SearchButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DriveComboBoxChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    BaseDelphiDir : String[128];
    SearchDir     : String[128];
  end;

var
  FindExpDialog: TFindExpDialog;

Type
  SourceFindExpert = class(TIExpert)
  public
    { Expert Style }
    function GetStyle: TExpertStyle; override;
    { Expert Strings }
    function GetIDString: string; override;
    function GetName: string; override;
    function GetComment: string; override;
    function GetGlyph: HBITMAP; override;
    function GetState: TExpertState; override;
    function GetMenuText: string; override;
    { Launch the Expert }
    procedure Execute; override;
  end;

  procedure Register;

implementation

uses IniFiles;

{$R *.DFM} 

procedure Register;
begin
  RegisterLibraryExpert(SourceFindExpert.Create);
end;

{-------- Method of the SourceFindExpert class --------}

function SourceFindExpert.GetStyle: TExpertStyle;
begin
  Result := esStandard
end;

function SourceFindExpert.GetIDString: String;
begin
  Result := 'Todd Miller.Source Find Expert'
end;

function SourceFindExpert.GetComment: String;
begin
  Result := '' { not needed for esStandard }
end;

function SourceFindExpert.GetGlyph: HBITMAP;
begin
  Result := 0 { not needed for esStandard }
end;

function SourceFindExpert.GetName: String;
begin
  Result := 'Source Find Expert'
end;

function SourceFindExpert.GetState: TExpertState;
begin
  Result := [esEnabled]
end;

function SourceFindExpert.GetMenuText: String;
begin
  Result := 'Source Find Expert (Todd Miller)'
end;

procedure SourceFindExpert.Execute;
begin
  if not Assigned(FindExpDialog) then
    FindExpDialog := TFindExpDialog.Create(Application);
  FindExpDialog.Show;
  FindExpDialog.SetFocus
end;

{-------- Method of the TFindExpDialog class --------}

procedure TFindExpDialog.SearchCriteriaClick(Sender: TObject);
begin

  case SearchCriteria.ItemIndex of
  0: SearchDir := BaseDelphiDir + 'SOURCE\VCL\';
  1: SearchDir := BaseDelphiDir + 'SOURCE\';
  2: SearchDir := BaseDelphiDir + 'DEMOS\';
  3: SearchDir := BaseDelphiDir + 'DOC\';
  4: SearchDir := BaseDelphiDir;
  5: SearchDir := UpperCase(DriveComboBox.Drive) + ':\';
  end;
  DriveComboBox.Visible := SearchCriteria.ItemIndex = 5;
  Caption := 'File Find Expert - ' + SearchDir;
  FoundList.Items.Clear;
  LoadButton.Enabled := False;
end;

procedure TFindExpDialog.CancelButtonClick(Sender: TObject);
begin
  Close;
end;

procedure TFindExpDialog.LoadButtonClick(Sender: TObject);
var
FileName : String;
begin
  if (FoundList.ItemIndex = -1) or not Assigned(ToolServices) then
     Exit;

  FileName := FoundList.Items[FoundList.ItemIndex];
  if UpperCase(ExtractFileExt(FileName)) = '.DPR' then begin
     if ToolServices.CloseProject then begin
        ToolServices.OpenProject(FileName)
     end;
  end else
    ToolServices.OpenFile(FileName);

  Close {done with the Find Source Expert Dialog}
end;

procedure TFindExpDialog.SearchButtonClick(Sender: TObject);
  procedure RecurseDirTree(NewPath : string);
  var
    SearchRec: TSearchRec;
  begin
  {The default extension is .PAS if one isn't present}
  with SearchFileName do
  if Pos('.', Text) = 0 then Text := Text + '.PAS';

  {Look for the search file name in the current directory}
  if FindFirst(NewPath + SearchFileName.Text, faAnyFile, SearchRec) = 0 then begin
     repeat
      {Check to make sure it is a normal file}
      if (SearchRec.Attr and (faHidden or faSysFile or FaVolumeID)) = 0 then begin
          FoundList.Items.Add(NewPath + SearchRec.Name);
          LoadButton.Enabled := True;
          Application.ProcessMessages;
       end;
     until FindNext(SearchRec) <> 0;
   end;
   {Go through the directories now}
   if FindFirst(NewPath + '*.*', faDirectory, SearchRec) = 0 then begin
      repeat
        if (SearchRec.Attr and faDirectory > 1)
           and (SearchRec.Name <> '.')
           and (SearchRec.Name <> '..') then
            RecurseDirTree(NewPath + SearchRec.Name + '\');
      until FindNext(SearchRec) <> 0;
    end;
  end;

begin
  LoadButton.Enabled := False;
  FoundList.Items.Clear;
  RecurseDirTree(SearchDir);
  {select the first found item by default}
  if FoundList.Items.Count > 0 then
     FoundList.ItemIndex := 0
  else
    FoundList.Items.Add('No File(s) Found');
end;

procedure TFindExpDialog.FormCreate(Sender: TObject);
var
  DelphiIni: TIniFile;
begin
  DelphiIni := TIniFile.Create('Delphi.Ini');
  try
  with DelphiIni do begin
    BaseDelphiDir := ReadString('Library', 'ComponentLibrary', 'ERROR');
    if BaseDelphiDir <> 'ERROR' then begin
       BaseDelphiDir := Copy(BaseDelphiDir, 1, Pos('BIN', BaseDelphiDir) - 1);
       {This sets up the default as the VCL directory}
       SearchDir := BaseDelphiDir + 'SOURCE\VCL\';
    end else begin
      MessageDlg('The Delphi.ini file could not be found - program terminating',mtError, [mbOk], 0);
      Application.terminate;
    end;
  end;
  finally
    DelphiIni.Free;
  end;
end;

procedure TFindExpDialog.DriveComboBoxChange(Sender: TObject);
begin
  SearchDir := UpperCase(DriveComboBox.Drive) + ':\';
  Caption := 'Source Find Expert - ' + SearchDir;
end;

procedure TFindExpDialog.FormShow(Sender: TObject);
begin
  SearchFileName.SetFocus;
end;

end.
